import time
from typing import List

import pygame
from pygame import Surface

from controllers.ControllerGame import ControllerGame
from views.components.ComponentGameObject import ComponentGameObject, sprite_width


class WindowMain:
    def __init__(self):
        pygame.init()
        self.game = ControllerGame.new_game()
        self.screen = pygame.display.set_mode(
            (self.game.map_size[0] * sprite_width, self.game.map_size[1] * sprite_width)
        )
        self.is_game_running = True
        self.game_components: List[ComponentGameObject] = []

    def show(self):
        # main game loop
        time_last = pygame.time.get_ticks()
        while self.is_game_running:
            # get delta seconds
            time_current = pygame.time.get_ticks()
            delta_milisec = time_current - time_last
            time_last = time_current

            # update
            self.update(delta_milisec)

            # draw
            self.draw()

            # update display
            pygame.display.flip()
            time.sleep(0.01)

    def sync_game_components(self):
        # remove missing game_components
        existing_game_objects = []
        for game_component in self.game_components:
            if game_component.game_object not in self.game.game_objects:
                self.game_components.remove(game_component)
            else:
                existing_game_objects.append(game_component.game_object)

        # add new game_components
        for game_object in self.game.game_objects:
            if game_object not in existing_game_objects:
                self.game_components.append(ComponentGameObject(game_object))


    def update(self, delta_milisec):
        self.sync_game_components()

        for game_component in self.game_components:
            game_component.update(delta_milisec)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.is_running = False
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.is_game_running = False

    def draw(self):
        self.screen.fill((0, 0, 0))
        for game_component in self.game_components:
            game_component.render(self.screen)